/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  
**
** Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: modtest.c,v 1.2 2002/08/06 03:58:03 bambi Exp $
**
*/

/*
** Module	: 
** Purpose	: 
** Exports	: 
** Depends Upon	: 
*/



/**************************************************************************
** STANDARD INCLUDES
**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

#include "ember.h"
#include "mod_msql.h"

e_extern	*module = msql_externs;

/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/



static int setupScript(script, argv, argc, skip)
	ember	*script;
	char	**argv;
	int	argc;
{
	e_stab	*sym;
	e_value	val;
	int	count;

	sym = eSymCreateVariable(script,"$argc");
	eSymSetIntValue(script, &sym->value, argc - skip);

	count = 0;
	sym = eSymCreateVariable(script,"$argv");
	while(count + skip < argc)
	{
		eSymSetTextValue(script, &val, argv[count + skip]);
		eSymSetArrayElement(script, &sym->value, &val, count);
		count++;
	}

        if (eAddModule(script, module, NULL) < 0)
                return(-1);
}


usage(msg)
	char	*msg;
{
	printf("\nModTest : Ember static module tester\n\nUsage error!\n");
	if(msg)
		printf("%s\n", msg);
	printf("\n");
	exit(1);
}

/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/

int main(argc, argv)
	int	argc;
	char	*argv[];
{
	ember	*script;
	e_stab	*symbol;
	char	c,
		*path = NULL,
		*outfile = NULL,
		*result = NULL;
	int	usageErr = 0,
		dFlag = 0,
		oFlag = 0,
		cFlag = 0,
		lFlag = 0;



	/*
	** Check out the command line args
	*/
	if (argc > 1 && *argv[1] == '-')
	{
		while((c = getopt(argc,argv,"dlco:")) != -1)
		{
			switch(c)
			{
				case 'd':
					if (dFlag)
						usageErr = 1;
					else
						dFlag = 1;
					break;
	
				case 'c':
					if (cFlag)
						usageErr = 1;
					else
						cFlag = 1;
					break;
	
				case 'l':
					if (lFlag)
						usageErr = 1;
					else
						lFlag = 1;
					break;
	
				case 'o':
					if (oFlag)
						usageErr = 1;
					else
					{
						oFlag = 1;
						outfile = optarg;
					}
					break;
	
				case '?':
					usageErr = 1;
					break;
			}
		}

		if (argc == optind < 1)
			usageErr = 1;
	}
	else
	{
		if (argc == 0)
			usageErr++;
		else
			optind = 1;
	}
	if (usageErr)
		usage(NULL);


	if (cFlag && lFlag)
	{
		usage("Can't use -l and -c at the same time!");
	}

	/*
	** Setup the given file name as the script source
	*/
	if (optind >= argc)
		usage("Missing script filename");
	path = argv[optind];
	script = eCreateScript();

	if (eFileSource(script, path) < 0)
	{
		fprintf(stderr,"Error opening file '%s'\n", 
			eGetSourceName(script));
		exit(1);
	}


	/*
	** Parse the script and report any errors
	*/
	setupScript(script, argv, argc, optind);

	if (eParseScript(script) < 0)
	{
		/* Syntax error etc */
		fprintf(stderr,"Error at line %d of script '%s'.\n",
			eGetLineNum(script), eGetSourceName(script));
		fprintf(stderr,"Error is '%s'\n\n", eGetErrorMsg(script));
		exit(1);
	}


	/*
	** If the dFlag is set then dump the symbol defs and the pcode
	** and exit
	*/
	if (dFlag)
	{
		/* Dump the pcode and symbol definitions */
		eDumpParseResult(script);
		exit(0);
	}

	/*
	** If the lFlag is set then we are creating a library file
	** from the parsed script file
	*/
	if (lFlag)
	{
		int	res;
		char	*cp;

		if (!oFlag)
		{
			outfile = malloc(strlen(path + 4));
			strcpy(outfile,path);
			cp = (char *)rindex(outfile,'.');
			if (cp)
				strcpy(cp,".lib");
			else
				strcat(cp,".lib");
		}
		res = eWriteLibrary(script, outfile);
		if (res < 0)
		{
			printf("\nError creating library!\n%s\n\n",
				script->errorMsg);
			exit(1);
		}
		exit(0);
	}


	/*
	** Run the script and report any errors
	*/
	if (eRunScript(script) < 0)
	{
		/* Syntax error etc */
		fprintf(stderr,"Runtime error at line %d of script '%s'.\n",
			eGetLineNum(script), eGetCurFileName(script));
		fprintf(stderr,"Error is '%s'\n\n", eGetErrorMsg(script));
		exit(1);
	}

        result = eSymGetVariableValue(script,"result");
        if (result)
                printf("\nResult = '%s'\n\n",result);

	eFreeScript(script);

	exit(0);
}
